<?php
/*
Plugin Name: Automatic WordPress Backup
Plugin URI: http://www.wordpressbackup.org/
Description: Automatically upload backups of important parts of your blog to Amazon S3
Version: 1.0.2
Author: Dan Coulter
Author URI: http://dancoulter.com/
*/ 

/*
 * Copyright 2009, Dan Coulter - http://co.deme.me | dan@dancoulter.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * @package automatic-wordpress-backup
 */

if ( !defined('WP_CONTENT_URL') )
	define( 'WP_CONTENT_URL', get_option('siteurl') . '/wp-content');
if ( !defined('WP_CONTENT_DIR') )
	define( 'WP_CONTENT_DIR', ABSPATH . 'wp-content' );
	
	
require_once 'wdc/wdc.class.php';

class cmAWB {
	/**
	 * Uses the init action to catch changes in the schedule and pass those on to the scheduler.
	 *
	 */
	function init() {
		if ( isset($_POST['s3b-schedule']) ) {
			$schedules = array(
				'daily' => 86400,
				'weekly' => 604800,
				'monthly' => 2592000,
			);
			
			if ( $_POST['s3b-schedule'] != 'disabled' ) {
				if ( get_option('s3b-schedule') != $_POST['s3b-schedule'] ) {
					wp_clear_scheduled_hook('s3-backup');
					wp_schedule_event(time() + $schedules[$_POST['s3b-schedule']], $_POST['s3b-schedule'], 's3-backup', array(false));
				}
			} else {
				wp_clear_scheduled_hook('s3-backup');
			}
			
			if ( $_POST['Submit'] == 'Save Changes and Backup Now' ) {
				wp_schedule_single_event(time(), 's3-backup', array(true));
			}
		}
		if ( isset($_POST['s3-new-bucket']) && !empty($_POST['s3-new-bucket']) ) {
			include_once 'S3.php';
			$settings = self::get_settings();
			$_POST['s3-new-bucket'] = strtolower($_POST['s3-new-bucket']);
			$s3 = new S3($settings['access-key'], $settings['secret-key']); 
			$s3->putBucket($_POST['s3-new-bucket']);
			$buckets = $s3->listBuckets();
			if ( is_array($buckets) && in_array($_POST['s3-new-bucket'], $buckets) ) {
				update_option('s3b-bucket', $_POST['s3-new-bucket']);
				$_POST['s3b-bucket'] = $_POST['s3-new-bucket'];
			} else {
				update_option('s3b-bucket', '');
				$_POST['s3b-bucket'] = '';
			}
		}
		if ( !get_option('s3b-bucket') ) add_action('admin_notices', array('cmAWB','newBucketWarning'));
	}
	
	function newBucketWarning() {
		echo "<div id='awb-warning' class='updated fade'><p><strong>".__('You need to select a valid S3 bucket.', 'automatic-wordpress-backup')."</strong> ".__('If you tried to create a new bucket, it may have been an invalid name.', 'automatic-wordpress-backup').' | <a href="' . self::get_plugin_page() . '">'.__('Plugin Settings', 'automatic-wordpress-backup').'</a></p></div>';

	}

	/**
	 * Return the filesystem path that the plugin lives in.
	 *
	 * @return string
	 */
	function getPath() {
		return dirname(__FILE__) . '/';
	}
	
	/**
	 * Returns the URL of the plugin's folder.
	 *
	 * @return string
	 */
	function getURL() {
		return WP_CONTENT_URL.'/plugins/'.basename(dirname(__FILE__)) . '/';
	}
	
	/**
	 * Returns the URL of the plugin's settings page.
	 *
	 * @return string
	 */
	function get_plugin_page($full = false) {
		global $wp_version;
		if ( !$full ) {
			return 'admin.php?page=' . plugin_basename(__FILE__);
		}
	}

	/**
     * Sets up the settings page
     *
     */
	function add_settings_page() {
		load_plugin_textdomain('automatic-wordpress-backup', cmAWB::getPath() . 'i18n');
		add_submenu_page('wdc-settings', __('Automatic Backup', 'automatic-wordpress-backup'), __('Automatic Backup', 'automatic-wordpress-backup'), 8, __FILE__, array('cmAWB', 'settings_page'));	
	}
	
	/**
	 * Generates the settings page
	 *
	 */
	function settings_page() {
		include_once 'S3.php';
		$sections = get_option('s3b-section');
		if ( !$sections ) {
			$sections = array();
		}
		$settings = self::get_settings();
		?>
			<script type="text/javascript">
				var ajaxTarget = "<?php echo self::getURL() ?>backup.ajax.php";
				var nonce = "<?php echo wp_create_nonce('automatic-wordpress-backup'); ?>";
			</script>
			<div class="wrap">
				<h2><?php _e('Automatic WordPress Backup', 'automatic-wordpress-backup') ?></h2>
				<?php if ( isset($_GET['updated']) ) : ?>
					<div class='updated fade'><p><?php _e('Settings saved.', 'automatic-wordpress-backup') ?></p></div>
					<?php if ( get_option('s3b-bucket') === '' ) : ?>
						<div class='error fade'><p><?php _e('No bucket was created.  You may have entered a bucket name that already exists on another account.', 'automatic-wordpress-backup') ?></p></div>
					<?php endif; ?>

				<?php endif; ?>
				<form method="post" action="options.php">
					<input type="hidden" name="action" value="update" />
					<?php wp_nonce_field('update-options'); ?>
					<input type="hidden" name="page_options" value="s3b-access-key,s3b-secret-key,s3b-bucket,s3b-section,s3b-schedule,awb-settings" />
					<p>
						<?php _e('AWS Access Key:', 'automatic-wordpress-backup') ?>
						<input type="text" name="awb-settings[access-key]" value="<?php echo (defined('AWB_ACCESS_KEY') ? 'DEFINED IN CONFIG' : $settings['access-key']); ?>" <?php if ( defined('AWB_ACCESS_KEY') ) echo 'readonly="readonly"' ?> />
					</p>
					<p>
						<?php _e('AWS Secret Key:', 'automatic-wordpress-backup') ?>
						<input type="password" name="awb-settings[secret-key]" value="<?php echo (defined('AWB_SECRET_KEY') ? 'DEFINED IN CONFIG' : $settings['secret-key']); ?>" <?php if ( defined('AWB_SECRET_KEY') ) echo 'readonly="readonly"' ?> />
					</p>
					<?php if ( $settings['access-key'] && $settings['secret-key'] ) : ?>
						<?php 
							$s3 = new S3($settings['access-key'], $settings['secret-key']); 
							$buckets = $s3->listBuckets();
						?>
						<p>
							<span style="vertical-align: middle;"><?php _e('S3 Bucket Name:', 'automatic-wordpress-backup') ?></span>
							<select name="s3b-bucket">
								<?php foreach ( $buckets as $b ) : ?>
									<option <?php if ( $b == get_option('s3b-bucket') ) echo 'selected="selected"' ?>><?php echo $b ?></option>
								<?php endforeach; ?>
							</select>
							
							<br />
							<span style="vertical-align: middle;"><?php _e('Or create a bucket:', 'automatic-wordpress-backup') ?></span>
							<input type="text" name="s3-new-bucket" id="new-s3-bucket" value="" />
							
						</p>
						<p>
							<span style="vertical-align: middle;"><?php _e('Backup schedule:', 'automatic-wordpress-backup') ?></span>
							<select name="s3b-schedule">
								<?php foreach ( array('Disabled','Daily','Weekly','Monthly') as $s ) : ?>
									<option value="<?php echo strtolower($s) ?>" <?php if ( strtolower($s) == get_option('s3b-schedule') ) echo 'selected="selected"' ?>><?php echo $s ?></option>
								<?php endforeach; ?>
							</select>
						</p>
						<p>
							<?php _e('Parts of your blog to back up', 'automatic-wordpress-backup') ?><br />
							<label for="s3b-section-config">
								<input <?php if ( in_array('config', $sections) ) echo 'checked="checked"' ?> type="checkbox" name="s3b-section[]" value="config" id="s3b-section-config" />
								<?php _e('Config file and htaccess', 'automatic-wordpress-backup') ?>
							</label><br />
							<label for="s3b-section-database">
								<input <?php if ( in_array('database', $sections) ) echo 'checked="checked"' ?> type="checkbox" name="s3b-section[]" value="database" id="s3b-section-database" />
								<?php _e('Database dump', 'automatic-wordpress-backup') ?>
							</label><br />
							<label for="s3b-section-themes">
								<input <?php if ( in_array('themes', $sections) ) echo 'checked="checked"' ?> type="checkbox" name="s3b-section[]" value="themes" id="s3b-section-themes" />
								<?php _e('Themes folder', 'automatic-wordpress-backup') ?>
							</label><br />
							<label for="s3b-section-plugins">
								<input <?php if ( in_array('plugins', $sections) ) echo 'checked="checked"' ?> type="checkbox" name="s3b-section[]" value="plugins" id="s3b-section-plugins" />
								<?php _e('Plugins folder', 'automatic-wordpress-backup') ?>
							</label><br />
							<?php do_action('s3b_sections') ?>
							<label for="s3b-section-uploads">
								<input <?php if ( in_array('uploads', $sections) ) echo 'checked="checked"' ?> type="checkbox" name="s3b-section[]" value="uploads" id="s3b-section-uploads" />
								<?php _e('Uploaded content', 'automatic-wordpress-backup') ?>
							</label><br />
						</p>
						<div>
							<label><input type="checkbox" id="awb-settings-cleanup" name="awb-settings[cleanup]" value="1" <?php if ( isset($settings['cleanup']) && $settings['cleanup']) echo 'checked="checked"' ?> /> <?php _e('Delete backups older than one month', 'automatic-wordpress-backup') ?></label>
							<div id="cleanup-settings" style="padding-left: 1em;">
								<label><input type="checkbox" name="awb-settings[cleanup-save-monthly]" value="1" <?php if ( isset($settings['cleanup-save-monthly']) && $settings['cleanup-save-monthly']) echo 'checked="checked"' ?> /> <?php _e('Keep a monthly backup for one year', 'automatic-wordpress-backup') ?></label><br />
								<label><input type="checkbox" name="awb-settings[cleanup-save-manual]" <?php if ( isset($settings['cleanup-save-manual']) && $settings['cleanup-save-manual']) echo 'checked="checked"' ?> value="1" /> <?php _e('Keep manual backups forever', 'automatic-wordpress-backup') ?></label>
							</div>
						</div>
					<?php endif; ?>
					<p class="submit">
						<input type="submit" name="Submit" value="<?php _e('Save Changes', 'automatic-wordpress-backup') ?>" />
						<input type="submit" id="backup-now" name="Submit" value="<?php _e('Save Changes and Backup Now', 'automatic-wordpress-backup') ?>" />
					</p>
					<p>
						<?php 
							$crons = _get_cron_array();
							foreach ( $crons as $cron ) {
								if ( isset($cron['s3-backup']) ) {
									$cron = current($cron['s3-backup']);
									if ( $cron['args'][0] ) {
										printf(__("Your manual backup has started. If it hasn't yet, it will show up below once it's completed <a href='%s'>the next time you view this page.</a>", 'automatic-wordpress-backup'), self::get_plugin_page() );									}
								}
							}
							
						?>
					</p>
				</form>
								
				<h3>Download recent backups</h3>
				<div id="backups">
					<?php 
						if ( get_option('s3b-bucket') ) {
							$backups = $s3->getBucket(get_option('s3b-bucket'), next(explode('//', get_bloginfo('siteurl'))));
							krsort($backups);
							$count = 0;
							foreach ( $backups as $key => $backup ) {
								$backup['label'] = sprintf(__('WordPress Backup from %s', 'automatic-wordpress-backup'), mysql2date(__('F j, Y h:i a'), date('Y-m-d H:i:s', $backup['time'] + (get_option('gmt_offset') * 3600))));
								
								if ( preg_match('|[0-9]{4}\.zip$|', $backup['name']) ) {
									$backup['label'] = sprintf(__('Manual WordPress Backup from %s', 'automatic-wordpress-backup'), mysql2date(__('F j, Y h:i a'), date('Y-m-d H:i:s', $backup['time'] + (get_option('gmt_offset') * 3600))));
								} elseif ( preg_match('|[0-9]{4}\.uploads\.zip$|', $backup['name']) ) {
									$backup['label'] = sprintf(__('Manual Uploads Backup from %s', 'automatic-wordpress-backup'), mysql2date(__('F j, Y h:i a'), date('Y-m-d H:i:s', $backup['time'] + (get_option('gmt_offset') * 3600))));
								} elseif ( preg_match('|\.uploads\.zip$|', $backup['name']) ) {
									$backup['label'] = sprintf(__('Uploads Backup from %s', 'automatic-wordpress-backup'), mysql2date(__('F j, Y h:i a'), date('Y-m-d H:i:s', $backup['time'] + (get_option('gmt_offset') * 3600))));
								}
								
								$backup = apply_filters('s3b-backup-item', $backup);
								
								if ( ++$count > 40 ) break;
								?>
									<div class="backup"><a href="<?php echo $s3->getObjectURL(get_option('s3b-bucket'), $backup['name']) ?>"><?php echo $backup['label'] ?></a></div>
								<?php
							}
						}
					?>
					<div class="backup">
					</div>
				</div>
				
			</div>
			<script type="text/javascript">
				;(function($){
				})(jQuery);
			</script>
		<?php
	}
	
	function get_settings() {
		$settings = get_option('awb-settings');
		if ( !$settings ) {
			register_setting('automatic-wordpress-backup', 'awb-settings');
			$settings = array();
		}
		$settings = array_merge(array(
			'cleanup' => 0,
			'cleanup-save-monthly' => 0,
			'cleanup-save-manual' => 0,
			'access-key' => null,
			'secret-key' => null,
		), $settings);
		if ( defined('AWB_ACCESS_KEY') ) $settings['access-key'] = AWB_ACCESS_KEY;
		if ( defined('AWB_SECRET_KEY') ) $settings['secret-key'] = AWB_SECRET_KEY;
		return $settings;
	}
	
	function backup($manual = false) {
		global $wpdb;
		require_once('S3.php');
		require_once(ABSPATH . '/wp-admin/includes/class-pclzip.php');
		
		$settings = self::get_settings();
		$s3 = new S3($settings['access-key'], $settings['secret-key']); 		
		$site =  next(explode('//', get_bloginfo('siteurl')));
		
		if ( isset($settings['cleanup']) && $settings['cleanup'] ) {
			$backups = $s3->getBucket(get_option('s3b-bucket'), $site);
			ksort($backups);
			$months = array();
			$_month = strtotime("-1 month");
			$_year = strtotime("-1 year");
			foreach ( $backups as $key => $backup ) {
				if ( preg_match('|[0-9]{4}\.zip$|', $backup['name']) ) {
					if ( !$settings['cleanup-save-manual'] && $_month > $backup['time'] ) {
						$s3->deleteObject(get_option('s3b-bucket'), $backup['name']);
					}
				} else {
					$month = date('Ym', $backup['time']);
					if ( $settings['cleanup-save-monthly'] && $_year < $backup['time'] && !in_array($month, $months) ) {
						$months[] = $month;
					} elseif ( $_month > $backup['time'] ) {
						$s3->deleteObject(get_option('s3b-bucket'), $backup['name']);
					}
				}
			}
		}

		$sections = get_option('s3b-section');
		if ( !$sections ) {
			$sections = array();
		}

		$file = WP_CONTENT_DIR . '/uploads/automatic-wordpress-backup.zip';

		$cwd = getcwd();
		chdir(ABSPATH);
		
		$backups = array();
		if ( in_array('config', $sections) ) {
			$backups[] = ABSPATH . 'wp-config.php';
			if ( is_file(ABSPATH . '.htaccess') ) {
				$backups[] = ABSPATH . '.htaccess';
			}
		}
		if ( in_array('database', $sections) ) {
			$tables = $wpdb->get_col("SHOW TABLES LIKE '" . $wpdb->prefix . "%'");
			$result = shell_exec('mysqldump --add-drop-table --single-transaction -h ' . DB_HOST . ' -u ' . DB_USER . ' --password="' . DB_PASSWORD . '" ' . DB_NAME . ' ' . implode(' ', $tables) . ' > ' .  WP_CONTENT_DIR . '/uploads/wp-s3-database-backup.sql');
			$backups[] = WP_CONTENT_DIR . '/uploads/wp-s3-database-backup.sql';
		}
		if ( in_array('themes', $sections) ) $backups[] = WP_CONTENT_DIR . '/themes';
		if ( in_array('plugins', $sections) ) $backups[] = WP_CONTENT_DIR . '/plugins';
		if ( in_array('uploads', $sections) ) $backups[] = WP_CONTENT_DIR . '/uploads';
		
		
		if ( !empty($backups) ) {
			foreach ( $backups as $key => $value ) {
				$backups[$key] = str_replace(ABSPATH, '', $value);
			}
			$result = shell_exec('zip -r ' . $file . ' ' . implode(' ', $backups));
			
			$upload = $s3->inputFile($file);
			if ( $manual ) {
				$s3->putObject($upload, get_option('s3b-bucket'), next(explode('//', get_bloginfo('siteurl'))) . '/' . date('Y-m-d-Hi') . '.zip');
			} else {
				$s3->putObject($upload, get_option('s3b-bucket'), next(explode('//', get_bloginfo('siteurl'))) . '/' . date('Y-m-d') . '.zip');
			}
			@unlink($file);
			@unlink(WP_CONTENT_DIR . '/uploads/wp-s3-database-backup.sql');
		}
		
		chdir($cwd);

	}
	
	function cron_schedules($schedules) {
		$schedules['weekly'] = array('interval'=>604800, 'display' => 'Once Weekly');
		$schedules['monthly'] = array('interval'=>2592000, 'display' => 'Once Monthly');
		return $schedules;
	}
	
	function settings_link($links) {
		$settings_link = '<a href="' . self::get_plugin_page() . '">' . __('Settings', 'automatic-wordpress-backup') . '</a>'; 
		array_unshift( $links, $settings_link ); 
		return $links; 
	}
	
	function wdc_plugins($plugins) {
		if ( is_array($plugins) ) {
			$plugins[] = 'Automatic WordPress Backup v1.0';
		}
		return $plugins;
	}
}

add_filter('cron_schedules', array('cmAWB', 'cron_schedules'));
add_action('admin_menu', array('cmAWB', 'add_settings_page'));
add_action('s3-backup', array('cmAWB', 'backup'), 1);
add_action('init', array('cmAWB', 'init'));
add_filter("plugin_action_links_" . plugin_basename(__FILE__), array('cmAWB', 'settings_link') ); 
add_filter('wdc_plugins', array('cmAWB', 'wdc_plugins'));
?>